# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/primitives.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17protos/primitives.proto\x12\x0b\x61ngr.protos\"\x87\x05\n\rCodeReference\x12:\n\x0btarget_type\x18\x01 \x01(\x0e\x32%.angr.protos.CodeReference.TargetType\x12<\n\x0coperand_type\x18\x02 \x01(\x0e\x32&.angr.protos.CodeReference.OperandType\x12\x35\n\x08location\x18\x03 \x01(\x0e\x32#.angr.protos.CodeReference.Location\x12\n\n\x02\x65\x61\x18\x04 \x01(\x04\x12\x0c\n\x04mask\x18\x05 \x01(\x04\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x0f\n\x07\x64\x61ta_ea\x18\x07 \x01(\x04\x12\x10\n\x08\x62lock_ea\x18\x08 \x01(\x04\x12\x10\n\x08stmt_idx\x18\t \x01(\x05\x12\x13\n\x0boperand_idx\x18\n \x01(\x05\x12:\n\x08ref_type\x18\x0b \x01(\x0e\x32(.angr.protos.CodeReference.ReferenceType\"=\n\nTargetType\x12\x0e\n\nCodeTarget\x10\x00\x12\x0e\n\nDataTarget\x10\x01\x12\x0f\n\x0bStackTarget\x10\x02\"~\n\x0bOperandType\x12\x14\n\x10ImmediateOperand\x10\x00\x12\x11\n\rMemoryOperand\x10\x01\x12\x1d\n\x19MemoryDisplacementOperand\x10\x02\x12\x16\n\x12\x43ontrolFlowOperand\x10\x03\x12\x0f\n\x0bOffsetTable\x10\x04\"&\n\x08Location\x12\x0c\n\x08Internal\x10\x00\x12\x0c\n\x08\x45xternal\x10\x01\"0\n\rReferenceType\x12\n\n\x06offset\x10\x00\x12\x08\n\x04read\x10\x01\x12\t\n\x05write\x10\x02\"k\n\x0bInstruction\x12\n\n\x02\x65\x61\x18\x01 \x01(\x04\x12\r\n\x05\x62ytes\x18\x02 \x01(\x0c\x12)\n\x05xrefs\x18\x03 \x01(\x0b\x32\x1a.angr.protos.CodeReference\x12\x16\n\x0elocal_noreturn\x18\x04 \x01(\x08\"`\n\x05\x42lock\x12\n\n\x02\x65\x61\x18\x01 \x01(\x04\x12.\n\x0cinstructions\x18\x02 \x01(\x0b\x32\x18.angr.protos.Instruction\x12\x0c\n\x04size\x18\x04 \x01(\r\x12\r\n\x05\x62ytes\x18\x05 \x01(\x0c\"\x95\x02\n\x10\x45xternalFunction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\n\n\x02\x65\x61\x18\x02 \x01(\x04\x12;\n\x02\x63\x63\x18\x03 \x01(\x0e\x32/.angr.protos.ExternalFunction.CallingConvention\x12\x12\n\nhas_return\x18\x04 \x01(\x08\x12\x11\n\tno_return\x18\x05 \x01(\x08\x12\x16\n\x0e\x61rgument_count\x18\x06 \x01(\x05\x12\x0f\n\x07is_weak\x18\x07 \x01(\x08\x12\x11\n\tprototype\x18\x08 \x01(\t\"G\n\x11\x43\x61llingConvention\x12\x11\n\rCallerCleanup\x10\x00\x12\x11\n\rCalleeCleanup\x10\x01\x12\x0c\n\x08\x46\x61stCall\x10\x02\"d\n\x10\x45xternalVariable\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\n\n\x02\x65\x61\x18\x02 \x01(\x04\x12\x0c\n\x04size\x18\x03 \x01(\r\x12\x0f\n\x07is_weak\x18\x04 \x01(\x08\x12\x17\n\x0fis_thread_local\x18\x05 \x01(\x08\"\xe5\x03\n\x04\x45\x64ge\x12\x0e\n\x06src_ea\x18\x01 \x01(\x04\x12\x0e\n\x06\x64st_ea\x18\x02 \x01(\x04\x12,\n\x08jumpkind\x18\x03 \x01(\x0e\x32\x1a.angr.protos.Edge.JumpKind\x12\x12\n\nis_outside\x18\x04 \x01(\x08\x12\x10\n\x08ins_addr\x18\x05 \x01(\x04\x12\x10\n\x08stmt_idx\x18\x06 \x01(\x03\x12)\n\x04\x64\x61ta\x18\x07 \x03(\x0b\x32\x1b.angr.protos.Edge.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\"\xfe\x01\n\x08JumpKind\x12\x13\n\x0fUnknownJumpkind\x10\x00\x12\n\n\x06\x42oring\x10\x01\x12\x08\n\x04\x43\x61ll\x10\x02\x12\n\n\x06Return\x10\x03\x12\x0e\n\nFakeReturn\x10\x04\x12\x0b\n\x07Syscall\x10\x05\x12\x0f\n\x0bSys_syscall\x10\x06\x12\x0e\n\nSys_int128\x10\x07\x12\x0c\n\x08NoDecode\x10\x08\x12\n\n\x06\x45mWarn\x10\t\x12\x11\n\rSigFPE_IntDiv\x10\n\x12\x0b\n\x07SigTRAP\x10\x0b\x12\x0b\n\x07SigSEGV\x10\x0c\x12\x0b\n\x07MapFail\x10\r\x12\x0b\n\x07NoRedir\x10\x0e\x12\r\n\tClientReq\x10\x0f\x12\r\n\tException\x10\x10\".\n\nBlockGraph\x12 \n\x05\x65\x64ges\x18\x01 \x03(\x0b\x32\x11.angr.protos.Edgeb\x06proto3')



_CODEREFERENCE = DESCRIPTOR.message_types_by_name['CodeReference']
_INSTRUCTION = DESCRIPTOR.message_types_by_name['Instruction']
_BLOCK = DESCRIPTOR.message_types_by_name['Block']
_EXTERNALFUNCTION = DESCRIPTOR.message_types_by_name['ExternalFunction']
_EXTERNALVARIABLE = DESCRIPTOR.message_types_by_name['ExternalVariable']
_EDGE = DESCRIPTOR.message_types_by_name['Edge']
_EDGE_DATAENTRY = _EDGE.nested_types_by_name['DataEntry']
_BLOCKGRAPH = DESCRIPTOR.message_types_by_name['BlockGraph']
_CODEREFERENCE_TARGETTYPE = _CODEREFERENCE.enum_types_by_name['TargetType']
_CODEREFERENCE_OPERANDTYPE = _CODEREFERENCE.enum_types_by_name['OperandType']
_CODEREFERENCE_LOCATION = _CODEREFERENCE.enum_types_by_name['Location']
_CODEREFERENCE_REFERENCETYPE = _CODEREFERENCE.enum_types_by_name['ReferenceType']
_EXTERNALFUNCTION_CALLINGCONVENTION = _EXTERNALFUNCTION.enum_types_by_name['CallingConvention']
_EDGE_JUMPKIND = _EDGE.enum_types_by_name['JumpKind']
CodeReference = _reflection.GeneratedProtocolMessageType('CodeReference', (_message.Message,), {
  'DESCRIPTOR' : _CODEREFERENCE,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.CodeReference)
  })
_sym_db.RegisterMessage(CodeReference)

Instruction = _reflection.GeneratedProtocolMessageType('Instruction', (_message.Message,), {
  'DESCRIPTOR' : _INSTRUCTION,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.Instruction)
  })
_sym_db.RegisterMessage(Instruction)

Block = _reflection.GeneratedProtocolMessageType('Block', (_message.Message,), {
  'DESCRIPTOR' : _BLOCK,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.Block)
  })
_sym_db.RegisterMessage(Block)

ExternalFunction = _reflection.GeneratedProtocolMessageType('ExternalFunction', (_message.Message,), {
  'DESCRIPTOR' : _EXTERNALFUNCTION,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.ExternalFunction)
  })
_sym_db.RegisterMessage(ExternalFunction)

ExternalVariable = _reflection.GeneratedProtocolMessageType('ExternalVariable', (_message.Message,), {
  'DESCRIPTOR' : _EXTERNALVARIABLE,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.ExternalVariable)
  })
_sym_db.RegisterMessage(ExternalVariable)

Edge = _reflection.GeneratedProtocolMessageType('Edge', (_message.Message,), {

  'DataEntry' : _reflection.GeneratedProtocolMessageType('DataEntry', (_message.Message,), {
    'DESCRIPTOR' : _EDGE_DATAENTRY,
    '__module__' : 'protos.primitives_pb2'
    # @@protoc_insertion_point(class_scope:angr.protos.Edge.DataEntry)
    })
  ,
  'DESCRIPTOR' : _EDGE,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.Edge)
  })
_sym_db.RegisterMessage(Edge)
_sym_db.RegisterMessage(Edge.DataEntry)

BlockGraph = _reflection.GeneratedProtocolMessageType('BlockGraph', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKGRAPH,
  '__module__' : 'protos.primitives_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.BlockGraph)
  })
_sym_db.RegisterMessage(BlockGraph)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EDGE_DATAENTRY._options = None
  _EDGE_DATAENTRY._serialized_options = b'8\001'
  _CODEREFERENCE._serialized_start=41
  _CODEREFERENCE._serialized_end=688
  _CODEREFERENCE_TARGETTYPE._serialized_start=409
  _CODEREFERENCE_TARGETTYPE._serialized_end=470
  _CODEREFERENCE_OPERANDTYPE._serialized_start=472
  _CODEREFERENCE_OPERANDTYPE._serialized_end=598
  _CODEREFERENCE_LOCATION._serialized_start=600
  _CODEREFERENCE_LOCATION._serialized_end=638
  _CODEREFERENCE_REFERENCETYPE._serialized_start=640
  _CODEREFERENCE_REFERENCETYPE._serialized_end=688
  _INSTRUCTION._serialized_start=690
  _INSTRUCTION._serialized_end=797
  _BLOCK._serialized_start=799
  _BLOCK._serialized_end=895
  _EXTERNALFUNCTION._serialized_start=898
  _EXTERNALFUNCTION._serialized_end=1175
  _EXTERNALFUNCTION_CALLINGCONVENTION._serialized_start=1104
  _EXTERNALFUNCTION_CALLINGCONVENTION._serialized_end=1175
  _EXTERNALVARIABLE._serialized_start=1177
  _EXTERNALVARIABLE._serialized_end=1277
  _EDGE._serialized_start=1280
  _EDGE._serialized_end=1765
  _EDGE_DATAENTRY._serialized_start=1465
  _EDGE_DATAENTRY._serialized_end=1508
  _EDGE_JUMPKIND._serialized_start=1511
  _EDGE_JUMPKIND._serialized_end=1765
  _BLOCKGRAPH._serialized_start=1767
  _BLOCKGRAPH._serialized_end=1813
# @@protoc_insertion_point(module_scope)
